/***********************************************************************
 * File: buffer_editor_test.c
 * 	(Extracted from
 *	   "$Id: buffer_editor_test.c,v 1.1 2008/08/09 19:08:37 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source file implements test program for `buffer_editor' API-s.
 *
 * Copyright (C) 2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include "buffer_editor.h"

int main(int ac, char **av)
{
	size_t	cmds_ix, pattern1s_ix, pattern2s_ix, cflags_ix
		, start_line_no_ix, end_line_no_ix
		, start_line_nos[] = {
			  0
			, 1
			, 3
			, (size_t)-1
			}
		, end_line_nos[] = {
			  0
			, 1
			, 5
			, 10
			, (size_t)-1
			};
	Buffer_edit_command_e	cmds[] = { 
			  Buffer_edit_cmd_extract
			, Buffer_edit_cmd_delete
			, Buffer_edit_cmd_substitute
			, Buffer_edit_cmd_substitute_only
			, Buffer_edit_cmd_general_substitute
			, Buffer_edit_cmd_general_substitute_only
			}
		;
	char	src[4096] =	"abc,def,ghi,jkl,mno\n"
				"123,456,789,012,345\n"
				"\n"
				"AbC,dEf,GhI,JkL,mNo\n"
				"i,ii,iii,iv,v,vi,vii\n"
				"		\n"
				"aBc,DeF,GhI,jKl,MnO\n"
				"I,II,III,IV,V,VI,VII\n"
				"                   \n"
				"0123xyz4567xYz89ab"
		, *pattern1s[] = {
				  "a(b|c[def)$"
				,
				"^abc"
				, "^$"
				, "^[ 	][ 	]*$"
				, "^[0-9].*[0-9]$"
				,
				",(def|iii),(.*),(jkl|vi),"
				, "xyz"
				}
		, *pattern2s[] = {
				  ""
				, "switch (def|iii) and (jkl|vi): {"
					Buffer_edit_pattern_backslash"0"
					"}=>{"
					Buffer_edit_pattern_backslash"3"
					","
					Buffer_edit_pattern_backslash"2"
					","
					Buffer_edit_pattern_backslash"1"
					Buffer_edit_pattern_backslash"9"
					"}"
				, "UWZYX"
				}
		, *dst = NULL
		, regcomp_errbuf[39]
		, *cmds_str[] = {
			  "Buffer_edit_cmd_extract"
			, "Buffer_edit_cmd_delete"
			, "Buffer_edit_cmd_substitute"
			, "Buffer_edit_cmd_substitute_only"
			, "Buffer_edit_cmd_general_substitute"
			, "Buffer_edit_cmd_general_substitute_only"
			}
		, *cflags_str[] = {
			  "REG_BASIC"
			, "REG_ICASE"
			, "REG_EXTENDED"
			, "(REG_ICASE | REG_EXTENDED)"
			}
		, **sv_av = av;
	int	cflags[] = {
			  REG_BASIC
			, REG_ICASE
			, REG_EXTENDED
			, (REG_ICASE | REG_EXTENDED)
			}
		, regcomp_errcode = REG_OKAY, sv_errno = 0
		, sv_ac = ac;

	ac = sv_ac; av = sv_av; /* for `splint' */

	printf("{{{ Test program for buffer_edit API-s starts...\n")
	; (void)fflush(stdout)
	;
	for (cmds_ix = 0
	; cmds_ix < (sizeof(cmds) / sizeof(cmds[0]))
	; ++cmds_ix) {
	for (pattern1s_ix = 0
	; pattern1s_ix < (sizeof(pattern1s) / sizeof(pattern1s[0]))
	; ++pattern1s_ix) {
	for (cflags_ix = 0
	; cflags_ix < (sizeof(cflags) / sizeof(cflags[0]))
	; ++cflags_ix) {
	for (pattern2s_ix = 0
	; pattern2s_ix < (sizeof(pattern2s) / sizeof(pattern2s[0]))
	; ++pattern2s_ix) {
	for (start_line_no_ix = 0
	; start_line_no_ix < (sizeof(start_line_nos)
					/ sizeof(start_line_nos[0]))
	; ++start_line_no_ix) {
	for (end_line_no_ix = 0
	; end_line_no_ix < (sizeof(end_line_nos)
					/ sizeof(end_line_nos[0]))
	; ++end_line_no_ix) {
		memset(regcomp_errbuf
			, (int)'@'
			, (sizeof(regcomp_errbuf) - 1)
			)
		; regcomp_errbuf[(sizeof(regcomp_errbuf) - 1)]	= '\0'
		;
		printf("{ (cmds_ix=%lu, pattern1s_ix=%lu, "
				"cflags_ix=%lu, pattern2s_ix=%lu, "
				"start_line_no_ix=%lu, "
				"end_line_no_ix=%lu)\n"
			"buffer_edit(\n"
			"src=[\n%s](len=%lu),\n"
			"\tstart_line_no=%lu,\n"
			"\tend_line_no=%lu,\n"
			"\tcmd=%d (%s),\n"
			"\tpattern1=[%s](len=%lu),\n"
			"\tcflags=0%04o (%s),\n"
			"\tpattern2=[%s](len=%lu),\n"
			"\tregcomp_errcode=%d,\n"
			"\tregcomp_errbuf=[%s](len=%lu),\n"
			"\tregcomp_errbuf_size=%lu) returns...\n"
			, (unsigned long)cmds_ix
			,	(unsigned long)pattern1s_ix
			,	(unsigned long)cflags_ix
			,	(unsigned long)pattern2s_ix
			,	(unsigned long)start_line_no_ix
			,	(unsigned long)end_line_no_ix
			, src, (unsigned long)strlen(src)
			, (unsigned long)(start_line_nos
						[start_line_no_ix]
						)
			, (unsigned long)(end_line_nos
						[end_line_no_ix]
						)
			, cmds[cmds_ix], cmds_str[cmds_ix]
			, pattern1s[pattern1s_ix]
			,	(unsigned long)strlen(pattern1s[
							pattern1s_ix])
			, (unsigned)cflags[cflags_ix]
			,	cflags_str[cflags_ix]
			, pattern2s[pattern2s_ix]
			,	(unsigned long)strlen(pattern2s[
						pattern2s_ix])
			, (regcomp_errcode = REG_OKAY)
			, regcomp_errbuf
			,	(unsigned long)strlen(regcomp_errbuf)
			, (unsigned long)sizeof(regcomp_errbuf)
			); (void)fflush(stdout)
			;
		errno	= 0
		; dst	= buffer_edit(src
				, start_line_nos[start_line_no_ix]
				, end_line_nos[end_line_no_ix]
				, cmds[cmds_ix]
				, pattern1s[pattern1s_ix]
				, cflags[cflags_ix]
				, pattern2s[pattern2s_ix]
				, '\0'
				, &regcomp_errcode
				, regcomp_errbuf
				, sizeof(regcomp_errbuf)
				)
		; sv_errno	= errno
		;
		if (dst == NULL) {
			printf("!!FAILED!! "); (void)fflush(stdout);
		}
		printf(	"dst@%p=[\n%s](len=%lu), "
			"and outputs regcomp_errcode=%d, "
			"regcomp_errbuf=[%s](len=%lu), "
			"errno=%d[%s].\n"
			"}\n"
			"\n"
			, dst
			, ((dst != NULL)? dst: "(n/a)")
			, ((dst != NULL)?
				  (unsigned long)strlen(dst)
				: 0UL
				)
			, regcomp_errcode
			, regcomp_errbuf
			,	(unsigned long)strlen(regcomp_errbuf)
			, sv_errno
			, ((sv_errno != 0)?
				  strerror(sv_errno)
				: "(n/a)"
				)
			); (void)fflush(stdout)
			;

		{	/* test: the other API-s... */
			void	*fsm = NULL;
			char	regcomp_errbuf2[128], *dst2 = NULL;
			int	result2, regcomp_errcode2 = 0;

			memset(regcomp_errbuf2
				, (int)'\0'
				, sizeof(regcomp_errbuf2)
				);
			if ((fsm = buffer_edit_regcomp(
					  pattern1s[pattern1s_ix]
					, cflags[cflags_ix]
					, pattern2s[pattern2s_ix]
					, '\\'
					, &regcomp_errcode2
					, regcomp_errbuf2
					, sizeof(regcomp_errbuf2)
					)) == NULL) {
				if (dst != NULL) {
					printf("!!FATAL!! "
						"buffer_edit_regcomp() "
						"returns NULL "
						"(errno=%d, "
						"regcomp_errcode2=%d, "
						"regcomp_errbuf2=[%s]"
						"(len=%lu)"
						").\n"
						, errno
						, regcomp_errcode2
						, regcomp_errbuf2
						, (unsigned long)strlen(
							regcomp_errbuf2
							)
						); (void)fflush(stdout);
				}
			}
			else if ((dst2 = buffer_edit_FSM(
					  src
					, start_line_nos
						[start_line_no_ix]
					, end_line_nos[end_line_no_ix]
					, cmds[cmds_ix]
					, fsm
					)) == NULL) {
				if (dst != NULL) {
					printf("!!FATAL!! "
						"buffer_edit_FSM() "
						"returns NULL "
						"(errno=%d).\n"
						, errno
						); (void)fflush(stdout);
				}
			}
			else if ((dst != NULL)
			&& strcmp(dst, dst2) != 0) {
				printf("!!FATAL!! results of "
					"buffer_edit_FSM() is differ "
					"from buffer_edit(). "
					"dst2=[\n%s](len=%lu)\n"
					, dst2
					, (unsigned long)strlen(dst2)
					); (void)fflush(stdout);
			}

			if ((dst2 != NULL)
			&& (result2 = buffer_edit_free(dst2)) != 0) {
				printf("!!FATAL!! buffer_edit() "
					"returns %d (errno=%d).\n"
					, result2, errno
					); (void)fflush(stdout);
			}

			if ((fsm != NULL)
			&& (result2 = buffer_edit_regfree(fsm)) != 0) {
				printf("!!FATAL!! "
					"buffer_edit_regfree() returns "
					"%d (errno=%d).\n"
					, result2, errno
					); (void)fflush(stdout);
			}
		}

		if ((dst != NULL)
		&& buffer_edit_free(dst) != 0) {
			printf("!!FAILED!! buffer_edit_free() returns "
				"error (errno=%d).\n"
				, errno
				); (void)fflush(stdout);
		}
	}}}}}}
	printf("}}} Test program for buffer_edit API-s finished!\n")
	; (void)fflush(stdout)
	;

	exit(0);
}
